# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from cliff import app
from cliff import commandmanager

import tuning_box


class TuningBoxApp(app.App):
    def __init__(self, client, **kwargs):
        super(TuningBoxApp, self).__init__(
            description='Tuning Box - configuration storage for your cloud',
            version=tuning_box.__version__,
            command_manager=commandmanager.CommandManager('tuning_box.cli'),
            **kwargs
        )
        self.client = client
